package Engine;

public class GameEngine implements Runnable {

    private static final int TARGET_FPS = 75;

    private final Window window;

    private final Thread gameLoopThread;

    private final Timer timer;

    private final IGameLogic gameLogic;

    private final MouseInput mouseInput;

    public GameEngine(String windowTitle, int width, int height, boolean vSync, IGameLogic gameLogic) throws Exception {
        gameLoopThread = new Thread(this, "GAME_LOOP_THREAD");
        window = new Window(windowTitle, width, height, vSync);
        mouseInput = new MouseInput();
        this.gameLogic = gameLogic;
        timer = new Timer();
    }

    public void start() {
        String osName = System.getProperty("os.name");
        if ( osName.contains("Mac") ) {
            gameLoopThread.run();
        } else {
            gameLoopThread.start();
        }
    }

    @Override
    public void run() {
        try {
            init();
            gameLoop();
        } catch (Exception excp) {
            excp.printStackTrace();
        } finally {
            cleanup();
        }
    }

    private void init() throws Exception {
        window.init();
        timer.init();
        mouseInput.init(window);
        gameLogic.init(window);
    }

    private void gameLoop() {
        while (!window.windowShouldClose()) {
            input();
            render();
            try{Thread.sleep(1);}
            catch (Exception ignored){}
            if ( !window.isvSync() ) {
                sync();
            }
        }
    }

    private void cleanup() {
        gameLogic.cleanup();                
    }
    
    private void sync() {
        float loopSlot = 1f / TARGET_FPS;
        double endTime = timer.getLastLoopTime() + loopSlot;
        while (timer.getTime() < endTime) {
            try {
                Thread.sleep(1);
            } catch (InterruptedException ignored) {
            }
        }
    }

    private void input() {
        gameLogic.input(window, mouseInput);
    }


    private void render() {
        gameLogic.render(window);
        window.update();
    }
}

class Timer {

    private double lastLoopTime;
    void init() {
        lastLoopTime = getTime();
    }

    double getTime() {
        return System.nanoTime() / 1000_000_000.0;
    }

    double getLastLoopTime() {
        return lastLoopTime;
    }
}
